package com.landray.kmss.sys.attachment.thirdSfs.util;

import com.landray.kmss.framework.plugin.core.config.IExtension;
import com.landray.kmss.framework.plugin.core.config.IExtensionPoint;
import com.landray.kmss.util.SpringBeanUtil;
import com.landray.kmss.sys.attachment.thirdSfs.bean.SysAttThirdValidatorService;
import com.landray.kmss.framework.service.plugin.Plugin;
import java.util.Iterator;
import com.landray.kmss.util.StringUtil;
import java.util.Collection;
import org.apache.commons.collections.CollectionUtils;
import com.alibaba.fastjson.JSONObject;
import java.util.ArrayList;
import org.slf4j.LoggerFactory;
import java.util.List;
import org.slf4j.Logger;

public class SysAttThirdValidatorUtil
{
    private static final Logger logger;
    private static boolean hasInit;
    private static List<ValidatorService> services;
    
    static {
        logger = LoggerFactory.getLogger(SysAttThirdValidatorUtil.class);
        SysAttThirdValidatorUtil.hasInit = false;
        SysAttThirdValidatorUtil.services = new ArrayList<ValidatorService>();
    }
    
    public static boolean validate(final String thirdType, final String modelName, final JSONObject param) {
        final List<ValidatorService> services = getServices();
        if (CollectionUtils.isEmpty((Collection)services)) {
            return false;
        }
        if (StringUtil.isNull(thirdType) || StringUtil.isNull(modelName)) {
            return false;
        }
        boolean result = false;
        for (final ValidatorService service : services) {
            try {
                if (!thirdType.equals(service.getKey()) || !modelName.equals(service.getModelName())) {
                    continue;
                }
                result = service.getService().validate(param);
            }
            catch (final Exception e) {
                SysAttThirdValidatorUtil.logger.error("校验出错, thirdType = {}, modelName = {}, param = {}", thirdType, modelName, param, e);
            }
        }
        return result;
    }
    
    public static List<ValidatorService> getServices() {
        if (!SysAttThirdValidatorUtil.hasInit) {
            init();
        }
        return SysAttThirdValidatorUtil.services;
    }
    
    private static List<ValidatorService> init() {
        if (SysAttThirdValidatorUtil.services.isEmpty()) {
            synchronized (SysAttThirdValidatorUtil.services) {
                if (SysAttThirdValidatorUtil.services.isEmpty()) {
                    final IExtensionPoint point = Plugin.getExtensionPoint("com.landray.kmss.sys.attachment.third.sfs.validator");
                    final IExtension[] extensions = point.getExtensions();
                    IExtension[] array;
                    for (int length = (array = extensions).length, i = 0; i < length; ++i) {
                        final IExtension extension = array[i];
                        if ("auth".equals(extension.getAttribute("name"))) {
                            final String key = Plugin.getParamValueString(extension, "key");
                            final String name = Plugin.getParamValueString(extension, "modelName");
                            final String serviceName = Plugin.getParamValueString(extension, "service");
                            final SysAttThirdValidatorService service = (SysAttThirdValidatorService)SpringBeanUtil.getBean(serviceName);
                            SysAttThirdValidatorUtil.services.add(new ValidatorService(key, name, service));
                        }
                    }
                    SysAttThirdValidatorUtil.hasInit = true;
                }
                monitorexit(SysAttThirdValidatorUtil.services);
            }
        }
        return SysAttThirdValidatorUtil.services;
    }
    
    static class ValidatorService
    {
        private String key;
        private String modelName;
        private SysAttThirdValidatorService service;
        
        public ValidatorService(final String key, final String name, final SysAttThirdValidatorService service) {
            this.key = key;
            this.modelName = name;
            this.service = service;
        }
        
        public String getKey() {
            return this.key;
        }
        
        public String getModelName() {
            return this.modelName;
        }
        
        public SysAttThirdValidatorService getService() {
            return this.service;
        }
    }
}
